from shared.gitmojis import *

mojis = [
    {
        "type": "fix",
        "icon": GJ_FIX,
        "code": ":bug:",
        "desc": "Fix a bug.",
    },
    {
        "type": "feat",
        "icon": GJ_FEAT,
        "code": ":sparkles:",
        "desc": "Introduce new features.",
    },
    {
        "type": "docs",
        "icon": GJ_DOCS,
        "code": ":memo:",
        "desc": "Add or update documentation.",
    },
    {
        "type": "style",
        "icon": GJ_STYLE,
        "code": ":art:",
        "desc": "Improve structure / format of the code.",
    },
    {
        "type": "refactor",
        "icon": GJ_REFACTOR,
        "code": ":recycle:",
        "desc": "Refactor code.",
    },
    {
        "type": "perf",
        "icon": GJ_PERF,
        "code": ":zap:",
        "desc": "Improve performance.",
    },
    {
        "type": "test",
        "icon": GJ_TEST,
        "code": ":white_check_mark:",
        "desc": "Add, update, or pass tests.",
    },
    {
        "type": "build",
        "icon": GJ_BUILD,
        "code": ":construction_worker:",
        "desc": "Add or update CI build system.",
    },
    {
        "type": "ci",
        "icon": GJ_CI,
        "code": ":green_heart:",
        "desc": "Fix CI Build.",
    },
    {
        "type": "revert",
        "icon": GJ_REVERT,
        "code": ":rewind:",
        "desc": "Revert changes.",
    },
    {
        "type": "dump",
        "icon": GJ_DUMP,
        "code": ":fire:",
        "desc": "Remove code or files.",
    },
    {
        "type": "hotfix",
        "icon": GJ_HOTFIX,
        "code": ":ambulance:",
        "desc": "Critical hotfix.",
    },
    {
        "type": "deploy",
        "icon": GJ_DEPLOY,
        "code": ":rocket:",
        "desc": "Deploy stuff.",
    },
    {
        "type": "ui",
        "icon": GJ_UI,
        "code": ":lipstick:",
        "desc": "Add or update the UI and style files.",
    },
    {
        "type": "init",
        "icon": GJ_INIT,
        "code": ":tada:",
        "desc": "Begin a project.",
    },
    {
        "type": "security",
        "icon": GJ_SECURITY,
        "code": ":lock:",
        "desc": "Fix security issues.",
    },
    {
        "type": "secret",
        "icon": GJ_SECRET,
        "code": ":closed_lock_with_key:",
        "desc": "Add or update secrets.",
    },
    {
        "type": "bump",
        "icon": GJ_BUMP,
        "code": ":bookmark:",
        "desc": "Release / Version tags.",
    },
    {
        "type": "fix-lint",
        "icon": GJ_FIX_LINT,
        "code": ":rotating_light:",
        "desc": "Fix compiler / linter warnings.",
    },
    {
        "type": "wip",
        "icon": GJ_WIP,
        "code": ":construction:",
        "desc": "Work in progress.",
    },
    {
        "type": "dep-drop",
        "icon": GJ_DEP_DROP,
        "code": ":arrow_down:",
        "desc": "Downgrade dependencies.",
    },
    {
        "type": "dep-bump",
        "icon": GJ_DEP_BUMP,
        "code": ":arrow_up:",
        "desc": "Upgrade dependencies.",
    },
    {
        "type": "pin",
        "icon": GJ_PIN,
        "code": ":pushpin:",
        "desc": "Pin dependencies to specific versions.",
    },
    {
        "type": "analytics",
        "icon": GJ_ANALYTICS,
        "code": ":chart_with_upwards_trend:",
        "desc": "Add or update analytics or track code.",
    },
    {
        "type": "dep-add",
        "icon": GJ_DEP_ADD,
        "code": ":heavy_plus_sign:",
        "desc": "Add a dependency.",
    },
    {
        "type": "dep-rm",
        "icon": GJ_DEP_RM,
        "code": ":heavy_minus_sign:",
        "desc": "Remove a dependency.",
    },
    {
        "type": "config",
        "icon": GJ_CONFIG,
        "code": ":wrench:",
        "desc": "Add or update configuration files.",
    },
    {
        "type": "script",
        "icon": GJ_SCRIPT,
        "code": ":hammer:",
        "desc": "Add or update development scripts.",
    },
    {
        "type": "lang",
        "icon": GJ_LANG,
        "code": ":globe_with_meridians:",
        "desc": "Internationalization and localization.",
    },
    {
        "type": "typo",
        "icon": GJ_TYPO,
        "code": ":pencil2:",
        "desc": "Fix typos.",
    },
    {
        "type": "poop",
        "icon": GJ_POOP,
        "code": ":poop:",
        "desc": "Write bad code that needs to be improved.",
    },
    {
        "type": "merge",
        "icon": GJ_MERGE,
        "code": ":twisted_rightwards_arrows:",
        "desc": "Merge branches.",
    },
    {
        "type": "package",
        "icon": GJ_PACKAGE,
        "code": ":package:",
        "desc": "Add or update compiled files or packages.",
    },
    {
        "type": "external",
        "icon": GJ_EXTERNAL,
        "code": ":alien:",
        "desc": "Update code due to external API changes.",
    },
    {
        "type": "resource",
        "icon": GJ_RESOURCE,
        "code": ":truck:",
        "desc": "Move or rename resources (e.g.: files, paths, routes).",
    },
    {
        "type": "license",
        "icon": GJ_LICENSE,
        "code": ":page_facing_up:",
        "desc": "Add or update license.",
    },
    {
        "type": "boom",
        "icon": GJ_BOOM,
        "code": ":boom:",
        "desc": "Introduce breaking changes.",
    },
    {
        "type": "asset",
        "icon": GJ_ASSET,
        "code": ":bento:",
        "desc": "Add or update assets.",
    },
    {
        "type": "accessibility",
        "icon": GJ_ACCESSIBILITY,
        "code": ":wheelchair:",
        "desc": "Improve accessibility.",
    },
    {
        "type": "source-docs",
        "icon": GJ_SOURCE_DOCS,
        "code": ":bulb:",
        "desc": "Add or update comments in source code.",
    },
    {
        "type": "beer",
        "icon": GJ_BEER,
        "code": ":beers:",
        "desc": "Write code drunkenly.",
    },
    {
        "type": "text",
        "icon": GJ_TEXT,
        "code": ":speech_balloon:",
        "desc": "Add or update text and literals.",
    },
    {
        "type": "db",
        "icon": GJ_DB,
        "code": ":card_file_box:",
        "desc": "Perform database related changes.",
    },
    {
        "type": "logs-add",
        "icon": GJ_LOGS_ADD,
        "code": ":loud_sound:",
        "desc": "Add or update logs.",
    },
    {
        "type": "logs-rm",
        "icon": GJ_LOGS_RM,
        "code": ":mute:",
        "desc": "Remove logs.",
    },
    {
        "type": "people",
        "icon": GJ_PEOPLE,
        "code": ":busts_in_silhouette:",
        "desc": "Add or update contributor(s).",
    },
    {
        "type": "ux",
        "icon": GJ_UX,
        "code": ":children_crossing:",
        "desc": "Improve user experience / usability.",
    },
    {
        "type": "arch",
        "icon": GJ_ARCH,
        "code": ":building_construction:",
        "desc": "Make architectural changes.",
    },
    {
        "type": "design",
        "icon": GJ_DESIGN,
        "code": ":iphone:",
        "desc": "Work on responsive design.",
    },
    {
        "type": "mock",
        "icon": GJ_MOCK,
        "code": ":clown_face:",
        "desc": "Mock things.",
    },
    {
        "type": "egg",
        "icon": GJ_EGG,
        "code": ":egg:",
        "desc": "Add or update an easter egg.",
    },
    {
        "type": "ignore",
        "icon": GJ_IGNORE,
        "code": ":see_no_evil:",
        "desc": "Add or update a .gitignore file.",
    },
    {
        "type": "snap",
        "icon": GJ_SNAP,
        "code": ":camera_flash:",
        "desc": "Add or update snapshots.",
    },
    {
        "type": "experiment",
        "icon": GJ_EXPERIMENT,
        "code": ":alembic:",
        "desc": "Perform experiments.",
    },
    {
        "type": "seo",
        "icon": GJ_SEO,
        "code": ":mag:",
        "desc": "Improve SEO.",
    },
    {
        "type": "types",
        "icon": GJ_TYPES,
        "code": ":label:",
        "desc": "Add or update types.",
    },
    {
        "type": "seed",
        "icon": GJ_SEED,
        "code": ":seedling:",
        "desc": "Add or update seed files.",
    },
    {
        "type": "flag",
        "icon": GJ_FLAG,
        "code": ":triangular_flag_on_post:",
        "desc": "Add, update, or remove feature flags.",
    },
    {
        "type": "catch",
        "icon": GJ_CATCH,
        "code": ":goal_net:",
        "desc": "Catch errors.",
    },
    {
        "type": "animation",
        "icon": GJ_ANIMATION,
        "code": ":dizzy:",
        "desc": "Add or update animations and transitions.",
    },
    {
        "type": "deprecation",
        "icon": GJ_DEPRECATION,
        "code": ":wastebasket:",
        "desc": "Deprecate code that needs to be cleaned up.",
    },
    {
        "type": "auth",
        "icon": GJ_AUTH,
        "code": ":passport_control:",
        "desc": "Work on code related to authorization, roles and permissions.",
    },
    {
        "type": "fix-simple",
        "icon": GJ_FIX_SIMPLE,
        "code": ":adhesive_bandage:",
        "desc": "Simple fix for a non-critical issue.",
    },
    {
        "type": "exploration",
        "icon": GJ_EXPLORATION,
        "code": ":monocle_face:",
        "desc": "Data exploration/inspection.",
    },
    {
        "type": "dead",
        "icon": GJ_DEAD,
        "code": ":coffin:",
        "desc": "Remove dead code.",
    },
    {
        "type": "test-fail",
        "icon": GJ_TEST_FAIL,
        "code": ":test_tube:",
        "desc": "Add a failing test.",
    },
    {
        "type": "logic",
        "icon": GJ_LOGIC,
        "code": ":necktie:",
        "desc": "Add or update business logic.",
    },
    {
        "type": "health",
        "icon": GJ_HEALTH,
        "code": ":stethoscope:",
        "desc": "Add or update healthcheck.",
    },
    {
        "type": "infra",
        "icon": GJ_INFRA,
        "code": ":bricks:",
        "desc": "Infrastructure related changes.",
    },
    {
        "type": "devxp",
        "icon": GJ_DEVXP,
        "code": ":technologist:",
        "desc": "Improve developer experience.",
    },
    {
        "type": "money",
        "icon": GJ_MONEY,
        "code": ":money_with_wings:",
        "desc": "Add sponsorships or money related infrastructure.",
    },
    {
        "type": "threading",
        "icon": GJ_THREADING,
        "code": ":thread:",
        "desc": "Add or update code related to multithreading or concurrency.",
    },
    {
        "type": "validation",
        "icon": GJ_VALIDATION,
        "code": ":safety_vest:",
        "desc": "Add or update code related to validation.",
    },
]
